@@ -30,9 +30,9 @@ if [ "${dask}" = "upstream_devel" ]; then
   pip install "dask[dataframe] @ git+https://github.com/dask/dask.git"
   pip install -U git+https://github.com/dask-contrib/dask-expr.git
 elif [ "${dask}" = "latest" ]; then
-  pip install dask[dataframe]
+  pip install "dask[dataframe]"
 else
-  pip install dask[dataframe]==${dask}
+  pip install "dask[dataframe]==${dask}"
 fi
 
 # additional dependencies needed for dask's s3 tests
