@@ -42,8 +42,8 @@
 #include "arrow/util/checked_cast.h"
 #include "arrow/util/decimal.h"
 #include "arrow/util/endian.h"
-#include "arrow/util/logging.h"
-#include "arrow/util/sort.h"
+#include "arrow/util/logging_internal.h"
+#include "arrow/util/sort_internal.h"
 #include "arrow/util/span.h"
 #include "arrow/visit_data_inline.h"
 #include "arrow/visit_type_inline.h"
@@ -95,7 +95,7 @@ class ArrayDataEndianSwapper {
   Status SwapType(const DataType& type) {
     RETURN_NOT_OK(VisitTypeInline(type, this));
     RETURN_NOT_OK(SwapChildren(type.fields()));
-    if (internal::HasValidityBitmap(type.id())) {
+    if (internal::may_have_validity_bitmap(type.id())) {
       // Copy null bitmap
       out_->buffers[0] = data_->buffers[0];
     }
@@ -125,7 +125,8 @@ class ArrayDataEndianSwapper {
     for (int64_t i = 0; i < length; i++) {
       out_data[i] = bit_util::ByteSwap(in_data[i]);
     }
-    return std::move(out_buffer);
+    // R build with openSUSE155 requires an explicit shared_ptr construction
+    return std::shared_ptr<Buffer>(std::move(out_buffer));
   }
 
   template <typename VALUE_TYPE>
@@ -151,57 +152,20 @@ class ArrayDataEndianSwapper {
     return Status::OK();
   }
 
-  Status Visit(const Decimal128Type& type) {
-    auto data = reinterpret_cast<const uint64_t*>(data_->buffers[1]->data());
+  template <typename T>
+  enable_if_decimal<T, Status> Visit(const T& type) {
+    using value_type = typename TypeTraits<T>::CType;
+    auto data = data_->buffers[1]->span_as<value_type>();
     ARROW_ASSIGN_OR_RAISE(auto new_buffer,
                           AllocateBuffer(data_->buffers[1]->size(), pool_));
-    auto new_data = reinterpret_cast<uint64_t*>(new_buffer->mutable_data());
-    // NOTE: data_->length not trusted (see warning above)
-    const int64_t length = data_->buffers[1]->size() / Decimal128Type::kByteWidth;
-    for (int64_t i = 0; i < length; i++) {
-      uint64_t tmp;
-      auto idx = i * 2;
-#if ARROW_LITTLE_ENDIAN
-      tmp = bit_util::FromBigEndian(data[idx]);
-      new_data[idx] = bit_util::FromBigEndian(data[idx + 1]);
-      new_data[idx + 1] = tmp;
-#else
-      tmp = bit_util::FromLittleEndian(data[idx]);
-      new_data[idx] = bit_util::FromLittleEndian(data[idx + 1]);
-      new_data[idx + 1] = tmp;
-#endif
-    }
-    out_->buffers[1] = std::move(new_buffer);
-    return Status::OK();
-  }
+    auto new_data = new_buffer->mutable_data_as<value_type>();
 
-  Status Visit(const Decimal256Type& type) {
-    auto data = reinterpret_cast<const uint64_t*>(data_->buffers[1]->data());
-    ARROW_ASSIGN_OR_RAISE(auto new_buffer, AllocateBuffer(data_->buffers[1]->size()));
-    auto new_data = reinterpret_cast<uint64_t*>(new_buffer->mutable_data());
-    // NOTE: data_->length not trusted (see warning above)
-    const int64_t length = data_->buffers[1]->size() / Decimal256Type::kByteWidth;
-    for (int64_t i = 0; i < length; i++) {
-      uint64_t tmp0, tmp1, tmp2;
-      auto idx = i * 4;
-#if ARROW_LITTLE_ENDIAN
-      tmp0 = bit_util::FromBigEndian(data[idx]);
-      tmp1 = bit_util::FromBigEndian(data[idx + 1]);
-      tmp2 = bit_util::FromBigEndian(data[idx + 2]);
-      new_data[idx] = bit_util::FromBigEndian(data[idx + 3]);
-      new_data[idx + 1] = tmp2;
-      new_data[idx + 2] = tmp1;
-      new_data[idx + 3] = tmp0;
-#else
-      tmp0 = bit_util::FromLittleEndian(data[idx]);
-      tmp1 = bit_util::FromLittleEndian(data[idx + 1]);
-      tmp2 = bit_util::FromLittleEndian(data[idx + 2]);
-      new_data[idx] = bit_util::FromLittleEndian(data[idx + 3]);
-      new_data[idx + 1] = tmp2;
-      new_data[idx + 2] = tmp1;
-      new_data[idx + 3] = tmp0;
-#endif
+    for (const value_type& v : data) {
+      auto bytes = v.ToBytes();
+      std::reverse(bytes.begin(), bytes.end());
+      memcpy(new_data++, bytes.data(), bytes.size());
     }
+
     out_->buffers[1] = std::move(new_buffer);
     return Status::OK();
   }
@@ -548,7 +512,7 @@ class NullArrayFactory {
   }
 
   Status Visit(const StructType& type) {
-    for (int i = 0; i < type_->num_fields(); ++i) {
+    for (int i = 0; i < type.num_fields(); ++i) {
       ARROW_ASSIGN_OR_RAISE(out_->child_data[i], CreateChild(type, i, length_));
     }
     return Status::OK();
@@ -865,7 +829,19 @@ class RepeatedArrayFactory {
   }
 
   Status Visit(const ExtensionType& type) {
-    return Status::NotImplemented("construction from scalar of type ", *scalar_.type);
+    // Retrieve the underlying storage scalar from the ExtensionScalar
+    const auto& ext_scalar = checked_cast<const ExtensionScalar&>(scalar_);
+    const auto& storage_scalar = ext_scalar.value;
+
+    // Create an array from the storage scalar
+    ARROW_ASSIGN_OR_RAISE(auto storage_array,
+                          MakeArrayFromScalar(*storage_scalar, length_, pool_));
+
+    auto ext_type = std::static_pointer_cast<ExtensionType>(ext_scalar.type);
+
+    out_ = type.WrapArray(ext_type, storage_array);
+
+    return Status::OK();
   }
 
   Result<std::shared_ptr<Buffer>> CreateUnionTypeCodes(int8_t type_code) {
