@@ -136,8 +136,8 @@ class PrepareTest < Test::Unit::TestCase
       {
         path: "c_glib/meson.build",
         hunks: [
-          ["-version = '#{@snapshot_version}'",
-           "+version = '#{@release_version}'"],
+          ["-    version: '#{@snapshot_version}',",
+           "+    version: '#{@release_version}',"],
         ],
       },
       {
@@ -161,6 +161,13 @@ class PrepareTest < Test::Unit::TestCase
            "+set(ARROW_VERSION \"#{@release_version}\")"],
         ],
       },
+      {
+        path: "cpp/meson.build",
+        hunks: [
+          ["-    version: '#{@snapshot_version}',",
+           "+    version: '#{@release_version}',"],
+        ],
+      },
       {
         path: "cpp/vcpkg.json",
         hunks: [
@@ -211,13 +218,6 @@ class PrepareTest < Test::Unit::TestCase
       ]
     end
     expected_changes += [
-      {
-        path: "js/package.json",
-        hunks: [
-          ["-  \"version\": \"#{@snapshot_version}\"",
-           "+  \"version\": \"#{@release_version}\""],
-        ],
-      },
       {
         path: "matlab/CMakeLists.txt",
         hunks: [
@@ -264,7 +264,8 @@ class PrepareTest < Test::Unit::TestCase
               "-<p><a href=\"../r/\">#{@previous_r_version} (release)</a></p>",
               "+<body><p><a href=\"../dev/r/\">#{@release_version}.9000 (dev)</a></p>",
               "+<p><a href=\"../r/\">#{@release_version} (release)</a></p>",
-              "+<p><a href=\"../#{@previous_compatible_version}/r/\">#{@previous_r_version}</a></p>",
+              "+<p><a href=\"../#{@previous_compatible_version}/r/\">" +
+              "#{@previous_r_version}</a></p>",
             ]
           ],
         },
